@@ -54,6 +54,7 @@ def calculate(temp_files, in_temp, stdout_, stderr_, forced=[]):
   compiler_rt_symbols = read_symbols(shared.path_from_root('system', 'lib', 'compiler-rt.symbols'))
   pthreads_symbols = read_symbols(shared.path_from_root('system', 'lib', 'pthreads.symbols'))
   wasm_libc_symbols = read_symbols(shared.path_from_root('system', 'lib', 'wasm-libc.symbols'))
+  html5_symbols = read_symbols(shared.path_from_root('system', 'lib', 'html5.symbols'))
 
   # XXX we should disable EMCC_DEBUG when building libs, just like in the relooper
 
@@ -252,6 +253,13 @@ def calculate(temp_files, in_temp, stdout_, stderr_, forced=[]):
     check_call([shared.PYTHON, shared.EMCC, shared.path_from_root('system', 'lib', 'gl.c'), '-o', o])
     return o
 
+  def create_html5(libname):
+    src_dir = shared.path_from_root('system', 'lib', 'html5')
+    files = []
+    for dirpath, dirnames, filenames in os.walk(src_dir):
+      files += map(lambda f: os.path.join(src_dir, f), filenames)
+    return build_libc(libname, files, ['-Oz'])
+
   def create_compiler_rt(libname):
     files = files_in_path(
       path_components=['system', 'lib', 'compiler-rt', 'lib', 'builtins'],
@@ -415,6 +423,7 @@ def calculate(temp_files, in_temp, stdout_, stderr_, forced=[]):
   system_libs = [('libcxx',        'a',  create_libcxx,      libcxx_symbols,      ['libcxxabi'], True),
                  ('libcxxabi',     'bc', create_libcxxabi,   libcxxabi_symbols,   ['libc'],      False),
                  ('gl',            'bc', create_gl,          gl_symbols,          ['libc'],      False),
+                 ('html5',         'bc', create_html5,       html5_symbols,       ['html5'],     False),
                  ('compiler-rt',   'a',  create_compiler_rt, compiler_rt_symbols, ['libc'],      False),
                  (dlmalloc_name(), 'bc', create_dlmalloc,    [],                  [],            False)]
 
@@ -569,7 +578,7 @@ class Ports(object):
       # note that tag **must** be the tag in sdl.py, it is where we store to (not where we load from, we just load the local dir)
       local_ports = os.environ.get('EMCC_LOCAL_PORTS')
       if local_ports:
-        local_ports = map(lambda pair: pair.split('='), local_ports.split(','))
+        local_ports = map(lambda pair: pair.split('=', 1), local_ports.split(','))
         for local in local_ports:
           if name == local[0]:
             path, subdir = local[1].split('|')
